@@ -4,7 +4,7 @@ Photonic::NonRetarded::SH
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 SYNOPSIS
 
@@ -156,7 +156,7 @@ polarization using the field (nrf) filter.
 =cut
 
 package Photonic::NonRetarded::SH;
-$Photonic::NonRetarded::SH::VERSION = '0.008';
+$Photonic::NonRetarded::SH::VERSION = '0.009';
 use namespace::autoclean;
 use PDL::Lite;
 use PDL::NiceSlice;
@@ -195,7 +195,7 @@ has 'field1'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
          lazy=>1, builder=>'_build_field1', 
          documentation=>'longitudinal field at fundamental');
 has 'field2'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
-         lazy=>1, builder=>'_build_field1', 
+         lazy=>1, builder=>'_build_field2', 
          documentation=>'longitudinal field at second harmonic');
 has 'epsL2'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
          writer=>'_epsL2', predicate=>'has_epsL2', 
@@ -215,7 +215,15 @@ has 'external_G'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
 has 'externalL_G'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
          lazy=>1, builder=>'_build_externalL_G', 
          documentation=>
-             'SH ext. longitudinal polarization in reciprocal space'); 
+             'SH ext. longitudinal polarization comp. in reciprocal space'); 
+has 'externalVecL_G'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
+         lazy=>1, builder=>'_build_externalVecL_G', 
+         documentation=>
+             'SH ext. longitudinal polarization proj. in recip. space'); 
+has 'externalVecL'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
+         lazy=>1, builder=>'_build_externalVecL', 
+         documentation=>
+             'SH ext. longitudinal polarization proj. in real space'); 
 has 'HP' =>(is=>'ro', isa=>'Photonic::NonRetarded::AllH', init_arg=>undef,
          lazy=>1, builder=>'_build_HP',
          documentation=>
@@ -397,6 +405,20 @@ sub _build_externalL_G {
     return $result;
 }
 
+sub _build_externalVecL_G {
+    my $self=shift;
+    my $extG=$self->externalL_G;
+    my $result=$self->nrf->nr->geometry->LC2Vec_G($extG);
+    return $result;
+}
+
+sub _build_externalVecL {
+    my $self=shift;
+    my $extG=$self->externalVecL_G;
+    my $result=GtoR($extG, $self->ndims, 1);
+    return $result;
+}
+    
 sub _build_HP { #build haydock states for P2
     my $self=shift;
     my $ext=$self->externalL_G;
@@ -414,7 +436,12 @@ sub _build_externalL_n {
     my $pol=$self->externalL_G;
     my $states=$self->HP->states;
     my $nh=$self->HP->iteration;
-    my @Pn=map HProd($states->[$_],$pol), 0..$nh-1;
+    # innecesario: \propto \delta_{n0}
+    #my @Pn=map HProd($states->[$_],$pol), 0..$nh-1;
+    my @Pn=map {0+0*i} 0..$nh-1;
+    #$Pn[0]=$pol->(:,(0),(0));
+    $Pn[0]=HProd($states->[0],$pol);
+    #print join " Pn ", @Pn[0..3], "\n";
     return PDL->pdl([@Pn])->complex;
 }
     
@@ -474,15 +501,71 @@ sub _build_P2 {
 
 sub _build_P2LMCalt {
     my $self=shift;
-    my $Pex=$self->external; #external 2w polarization
-    #normalized longitudinal field.
-    my $field=$self->field2/$self->nrf->nr->geometry->npoints; 
-    #RorI xory nx ny
-    # <0|epsilon^{-1}_{LL}|P^{ex}>
-    my $prod=$field*$Pex; #Notice I don't conjugate. Epsilon is no hermitian
-    $prod=$prod->real->mv(0,-1)->clump(-2)->sumover->complex;
-    die "Strange, no epsilonL2" unless $self->has_epsL2;
-    $prod/=$self->epsL2;
+    my $PexL_G=$self->externalL_G; #external long 2w polarization
+    my $PexM=$self->external_G->(:,:,(0),(0)); #macroscopic external.
+    my $ndims=$self->nrf->nr->geometry->ndims;
+    my $nelem=$self->nrf->nr->geometry->npoints;
+    my $k=$self->nrf->nr->geometry->Direction0;
+    my $GNorm=$self->nrf->nr->geometry->GNorm;
+    my $epsA2=$self->epsA2;
+    my $u2=$self->u2;
+    my $B=$self->nrf->nr->geometry->B;
+    my $as=$self->nrf->nr->as;
+    my $b2s=$self->nrf->nr->b2s;
+    my $bs=$self->nrf->nr->bs;
+    my $states=$self->nrf->nr->states;
+    my $nh=$self->nrf->nh; #desired number of Haydock terms
+    #don't go beyond available values.
+    $nh=$self->nrf->nr->iteration if $nh>=$self->nrf->nr->iteration;
+    # calculate using linpack for tridiag system
+    # solve \epsilon^LL \vec E^L=|0>.
+    my $diag=$self->u2->Cconj->complex - PDL->pdl([@$as])->(0:$nh-1);
+    my $subdiag=-PDL->pdl(@$bs)->(0:$nh-1)->r2C;
+    # rotate complex zero from first to last element.
+    my $supradiag=$subdiag->real->mv(0,-1)->rotate(-1)->mv(-1,0)->complex;
+    my $rhs=PDL->zeroes($nh);
+    $rhs->((0)).=1;
+    $rhs=$rhs->r2C;
+    my ($phi_n, $info)= cgtsl($subdiag, $diag, $supradiag, $rhs); 
+    die "Error solving tridiag system" unless $info == 0;
+    my $phi_G=linearCombine([$phi_n->dog], $states);
+    my $Pphi=$k*(1-$epsA2)*$u2/$epsA2*HProd($phi_G, $PexL_G);
+
+    my $beta_G=RtoG($B*GtoR($states->[0],$ndims,0), $ndims,0);
+    #my $beta_G=RtoG(GtoR($states->[0],$ndims,0), $ndims,0);
+    my $betaV_G=$beta_G->(,*1)*$GNorm;
+    #my $betaV_G=$beta_G->(,*1)*$k;
+    my $betaV_n=PDL->pdl(
+	[map {HProd($betaV_G,$states->[$_]->(,*1), 1)} (0..$nh-1)]
+	)->complex;
+    #my $tmp=HProd($betaV_G->(,(1),:,:), $states->[200]);
+    #print $tmp, "==?", $betaV_n->(:,(1), (200)), "\n";
+    
+    my @Ppsi;
+    foreach(0..$ndims-1){
+	my ($psi_n, $psiinfo)= 
+	    cgtsl($subdiag, $diag, $supradiag, $betaV_n->(:,($_),:)); 
+	die "Error solving tridiag system" unless $psiinfo == 0;
+	# RorI nx ny .... cartesian
+	#print "psi_n[$_]=$psi_n\n";
+	my $psi_G=linearCombine([$psi_n->dog], $states);
+	my $Ppsi=HProd($psi_G, $PexL_G);
+	push @Ppsi, $Ppsi;
+    }
+    my $Ppsi=PDL->pdl(@Ppsi)->complex;
+    #my $P2M=$Pphi+$Ppsi;
+    my $P2M=$Pphi+$Ppsi+$PexM*$nelem; # Unnormalize Pex !!
+    #my $P2M=$Pphi;#+$Ppsi;
+    #my $P2M=$Ppsi;#+$Ppsi;
+    #my $P2M=PDL->pdl(@P2M)->complex;
+    # RorI nx ny .... cartesian
+    #my $psiV_n=PDL->pdl(@psi_ns);
+    #my $psi_G=linearCombine([$psiV_n->dog], $states);
+    #my $Pphi=(1-$epsA2)*$u2/$epsA2*HProd($phi_G, $PexL_G);
+    #my $Ppsi=HProd($psi_G, $PexL_G);
+    #my $P2M=$k*($Pphi+$Ppsi);
+    return $P2M->(,,*1,*1);
+    #return $prod->(,*1)*$self->nrf->nr->geometry->Direction0;
 }
 
 sub _build_u1 {
